#![allow(clippy::all)]
pub mod clusterclasses;
pub mod clusterresourcesets;
pub mod clusters;
pub mod kubeadmconfigtemplates;
pub mod kubeadmcontrolplane;
pub mod kubeadmcontrolplanetemplates;
pub mod machines;
pub mod openstackclustertemplates;
pub mod openstackmachinetemplates;
